// agena.c - sample pass through GINA dll

#define WINVER 0x500

#include <windows.h>
#include <stdio.h>
#include <winwlx.h>
#include "agena.h"

PWLX_DISPATCH_VERSION_1_3  pWinlogon;
DWORD  dwVersion;

// 1.0 Entrypoints
FPWLXNEGOTIATE             pWlxNegotiate = NULL;
FPWLXINITIALIZE            pWlxInitialize = NULL;
FPWLXDISPLAYSASNOTICE      pWlxDisplaySASNotice = NULL;
FPWLXLOGGEDOUTSAS          pWlxLoggedOutSAS = NULL;
FPWLXACTIVATEUSERSHELL     pWlxActivateUserShell = NULL;
FPWLXLOGGEDONSAS           pWlxLoggedOnSAS = NULL;
FPWLXDISPLAYLOCKEDNOTICE   pWlxDisplayLockedNotice = NULL;
FPWLXWKSTALOCKEDSAS        pWlxWkstaLockedSAS = NULL;
FPWLXISLOCKOK              pWlxIsLockOk = NULL;
FPWLXISLOGOFFOK            pWlxIsLogoffOk = NULL;
FPWLXLOGOFF                pWlxLogoff = NULL;
FPWLXSHUTDOWN              pWlxShutdown = NULL;

// 1.1 Entrypoints
FPWLXSTARTAPPLICATION      pWlxStartApplication = NULL;
FPWLXSCREENSAVERNOTIFY     pWlxScreenSaverNotify = NULL;

#if (WINVER >= 0x0500)
// 1.3 Entrypoints
FPWLXNETWORKPROVIDERLOAD   pWlxNetworkProviderLoad = NULL; 
FPWLXDISPLAYNOTICE         pWlxDisplayNotice = NULL;
FPWLXDISPLAYSTATUSMESSAGE  pWlxDisplayStatusMessage = NULL;
FPWLXGETSTATUSMESSAGE      pWlxGetStatusMessage = NULL;
FPWLXREMOVESTATUSMESSAGE   pWlxRemoveStatusMessage = NULL;
#endif

//-----------------------------------------------------------------
BOOL AgenaUninstall(void)
{
    HKEY hKey;

    OutputDebugString("Agena!AgenaUninstall\n");
    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, REGKEY_WINLOGON, 0,
                     KEY_ALL_ACCESS, &hKey) != ERROR_SUCCESS) {
        return FALSE;
    }

    RegDeleteValue(hKey, REGVALUE_GINADLL);
    RegCloseKey(hKey);
    return TRUE;
}

//-----------------------------------------------------------------
BOOL AgenaInitialize(void)
{
    HINSTANCE hDll;

    OutputDebugString("Agena!AgenaInitialize\n");

    // Load MSGINA.DLL.   
    if(!(hDll = LoadLibrary(DEFAULT_GINA_PATH))) {
        return FALSE;
    }

    // Get pointers to all of WLX functions in the default GINA.
    
    pWlxNegotiate            = (FPWLXNEGOTIATE)GetProcAddress(
                               hDll, "WlxNegotiate");
    pWlxInitialize           = (FPWLXINITIALIZE)GetProcAddress(
                               hDll, "WlxInitialize");
    pWlxDisplaySASNotice     = (FPWLXDISPLAYSASNOTICE)GetProcAddress(
                               hDll, "WlxDisplaySASNotice");
    pWlxLoggedOutSAS         = (FPWLXLOGGEDOUTSAS)GetProcAddress(
                               hDll, "WlxLoggedOutSAS");
    pWlxActivateUserShell    = (FPWLXACTIVATEUSERSHELL)GetProcAddress(
                               hDll, "WlxActivateUserShell");
    pWlxLoggedOnSAS          = (FPWLXLOGGEDONSAS)GetProcAddress(
                               hDll, "WlxLoggedOnSAS");
    pWlxDisplayLockedNotice  = (FPWLXDISPLAYLOCKEDNOTICE)
                               GetProcAddress(
                               hDll, "WlxDisplayLockedNotice");
    pWlxIsLockOk             = (FPWLXISLOCKOK)GetProcAddress(
                               hDll, "WlxIsLockOk");
    pWlxWkstaLockedSAS       = (FPWLXWKSTALOCKEDSAS)GetProcAddress(
                               hDll, "WlxWkstaLockedSAS");
    pWlxIsLogoffOk           = (FPWLXISLOGOFFOK)GetProcAddress(
                               hDll, "WlxIsLogoffOk");
    pWlxLogoff               = (FPWLXLOGOFF)GetProcAddress(
                               hDll, "WlxLogoff");
    pWlxShutdown             = (FPWLXSHUTDOWN)GetProcAddress(
                               hDll, "WlxShutdown");
 
    if (!pWlxNegotiate || !pWlxInitialize || 
        !pWlxDisplaySASNotice || !pWlxLoggedOutSAS || 
        !pWlxActivateUserShell || !pWlxLoggedOnSAS || 
        !pWlxDisplayLockedNotice || !pWlxIsLockOk || 
        !pWlxWkstaLockedSAS || !pWlxLogoff || !pWlxShutdown) {
        return FALSE; 
    }

    // 1.1 entrypoints (only for NT 4.0 and later)

    pWlxStartApplication     = (FPWLXSTARTAPPLICATION)GetProcAddress(
                               hDll, "WlxStartApplication");
    pWlxScreenSaverNotify    = (FPWLXSCREENSAVERNOTIFY)GetProcAddress(
                               hDll, "WlxScreenSaverNotify");
    
    // 1.3 entrypoints (only for NT 5.0 and later)

    pWlxNetworkProviderLoad  = (FPWLXNETWORKPROVIDERLOAD)
                               GetProcAddress(
                               hDll, "WlxNetworkProviderLoad");
    pWlxDisplayNotice        = (FPWLXDISPLAYNOTICE)GetProcAddress(
                               hDll, "WlxDisplayNotice");
    pWlxDisplayStatusMessage = (FPWLXDISPLAYSTATUSMESSAGE)
                               GetProcAddress(
                               hDll, "WlxDisplayStatusMessage");
    pWlxGetStatusMessage     = (FPWLXGETSTATUSMESSAGE)GetProcAddress(
                               hDll, "WlxGetStatusMessage");  
    pWlxRemoveStatusMessage  = (FPWLXREMOVESTATUSMESSAGE)
                               GetProcAddress(
                               hDll, "WlxRemoveStatusMessage");   
    return TRUE;
}

//-----------------------------------------------------------------
BOOL WINAPI
WlxNegotiate(DWORD dwWinlogonVersion, DWORD *pdwDllVersion)
{
    TCHAR szDbg[MAX_PATH];
    
    wsprintf(szDbg, TEXT("Agena!WlxNegotiate (%x)\n"), 
             dwWinlogonVersion);
    OutputDebugString(szDbg);

    dwVersion = dwWinlogonVersion;
    if (dwWinlogonVersion > AGENA_VERSION) {
        AgenaUninstall();
        return FALSE;
    }

    if (!AgenaInitialize()) {
        return FALSE;
    }
    return pWlxNegotiate(dwWinlogonVersion, pdwDllVersion);
}

//-----------------------------------------------------------------
BOOL WINAPI
WlxInitialize(LPWSTR lpWinsta, HANDLE hWlx, PVOID pvReserved,
              PVOID pWinlogonFunctions, PVOID *pWlxContext)
{
    pWinlogon = pWinlogonFunctions;

    OutputDebugString("Agena!WlxInitialize\n");
    return pWlxInitialize(lpWinsta, hWlx, pvReserved, 
                          pWinlogonFunctions, pWlxContext);
}

//-----------------------------------------------------------------
VOID WINAPI
WlxDisplaySASNotice(PVOID pWlxContext)
{
    OutputDebugString("Agena!WlxDisplaySASNotice\n");
    pWlxDisplaySASNotice(pWlxContext);
}

//-----------------------------------------------------------------
int WINAPI
WlxLoggedOutSAS(PVOID pWlxContext, DWORD dwSasType,
                PLUID pAuthenticationId, PSID pLogonSid,
                PDWORD pdwOptions, PHANDLE phToken,
                PWLX_MPR_NOTIFY_INFO pMprNotifyInfo,
                PVOID *pProfile)
{
    OutputDebugString("Agena!WlxLoggedOutSAS\n");
    return pWlxLoggedOutSAS(pWlxContext, dwSasType,
                            pAuthenticationId, pLogonSid,
                            pdwOptions, phToken, pMprNotifyInfo,
                            pProfile);
}

//-----------------------------------------------------------------
BOOL WINAPI
WlxActivateUserShell(PVOID pWlxContext, PWSTR pszDesktopName,
                     PWSTR pszMprLogonScript,
                     PVOID pEnvironment)
{
    OutputDebugString("Agena!WlxActivateUserShell\n");
    return pWlxActivateUserShell(pWlxContext, pszDesktopName,
                                 pszMprLogonScript, pEnvironment);
}

//-----------------------------------------------------------------
int WINAPI
WlxLoggedOnSAS(PVOID pWlxContext, DWORD dwSasType, PVOID pReserved)
{
    OutputDebugString("Agena!WlxLoggedOnSAS\n");
    return pWlxLoggedOnSAS(pWlxContext, dwSasType, pReserved);
}

//-----------------------------------------------------------------
VOID WINAPI
WlxDisplayLockedNotice(PVOID pWlxContext)
{
    OutputDebugString("Agena!WlxDisplayLockedNotice\n");
    pWlxDisplayLockedNotice(pWlxContext);
}

//-----------------------------------------------------------------
BOOL WINAPI
WlxIsLockOk(PVOID pWlxContext)
{
    OutputDebugString("Agena!WlxIsLockOk\n");
    return pWlxIsLockOk(pWlxContext);
}

//-----------------------------------------------------------------
int WINAPI
WlxWkstaLockedSAS(PVOID pWlxContext, DWORD dwSasType)
{
    OutputDebugString("Agena!WlxWkstaLockedSAS\n");
    return pWlxWkstaLockedSAS(pWlxContext, dwSasType);
}

//-----------------------------------------------------------------
BOOL WINAPI
WlxIsLogoffOk(PVOID pWlxContext)
{
    OutputDebugString("Agena!WlxLogoffOk\n");
    return pWlxIsLogoffOk(pWlxContext);
}

//-----------------------------------------------------------------
VOID WINAPI
WlxLogoff(PVOID pWlxContext)
{
    OutputDebugString("Agena!WlxLogoff\n");
    pWlxLogoff(pWlxContext);
}

//-----------------------------------------------------------------
VOID WINAPI
WlxShutdown(PVOID pWlxContext, DWORD ShutdownType)
{
    OutputDebugString("Agena!WlxShutdown\n");
    pWlxShutdown(pWlxContext, ShutdownType);
}

//-----------------------------------------------------------------
//                    1.1 Entrypoints
//-----------------------------------------------------------------
BOOL WINAPI
WlxScreenSaverNotify(PVOID pWlxContext, BOOL *pSecure)
{
    OutputDebugString("Agena!WlxScreenSaverNotify\n");
    if(pWlxScreenSaverNotify) {
        return pWlxScreenSaverNotify(pWlxContext, pSecure);
    }
    return TRUE;
}

//-----------------------------------------------------------------
BOOL WINAPI
WlxStartApplication(PVOID pWlxContext, PWSTR pszDesktopName,
                    PVOID pEnvironment, PWSTR pszCmdLine)
{
    OutputDebugString("Agena!WlxStartApplication\n");
    if(pWlxStartApplication) {
        return pWlxStartApplication(pWlxContext, pszDesktopName,
                                    pEnvironment, pszCmdLine);
    }
    return FALSE;
}

//-----------------------------------------------------------------
//                    1.3 Entrypoints
//-----------------------------------------------------------------
#if (WINVER >= 0x0500)
BOOL WINAPI
WlxNetworkProviderLoad(PVOID pWlxContext,
                       PWLX_MPR_NOTIFY_INFO pNprNotifyInfo)
{
    OutputDebugString("Agena!WlxNetworkProviderLoad\n");
    if (pWlxNetworkProviderLoad) {
        return pWlxNetworkProviderLoad(pWlxContext, pNprNotifyInfo);
    }
    return FALSE;
}

//-----------------------------------------------------------------
BOOL WINAPI
WlxDisplayNotice(PVOID pWlxContext, WLX_NOTICE_TYPE NoticeType)
{
    OutputDebugString("Agena!WlxDisplayNotice\n");
    if (pWlxDisplayNotice) {
        return pWlxDisplayNotice(pWlxContext, NoticeType);
    }
    return FALSE;
}

//-----------------------------------------------------------------
BOOL WINAPI
WlxDisplayStatusMessage(PVOID pWlxContext, HDESK hDesktop,
                        PWSTR pTitle, PWSTR pMessage)
{
    OutputDebugString("Agena!WlxDisplayStatusMessage\n");
    if (pWlxDisplayStatusMessage) {
        return pWlxDisplayStatusMessage(pWlxContext, hDesktop,
                                        pTitle, pMessage);
    }
    return FALSE;
}

//-----------------------------------------------------------------
BOOL WINAPI
WlxGetStatusMessage(PVOID pWlxContext, PWSTR pMessage,
                    DWORD dwBufferSize)
{
    OutputDebugString("Agena!WlxGetStatusMessage\n");
    if (pWlxGetStatusMessage) {
        return pWlxGetStatusMessage(pWlxContext, pMessage,
                                    dwBufferSize);
    }
	return FALSE;
}

//-----------------------------------------------------------------
BOOL WINAPI
WlxRemoveStatusMessage(PVOID pWlxContext)
{
    OutputDebugString("Agena!WlxRemoveStatusMessage\n");
    if (pWlxRemoveStatusMessage) {
        return pWlxRemoveStatusMessage(pWlxContext);
    }
    return FALSE;
}
#endif
