#ifndef _TEMPLATE_UTILITY_ADAPTATIONS_
#define _TEMPLATE_UTILITY_ADAPTATIONS_

#include <afxtempl.h> //!!PUT this into the precompiled header

//////////////////////////////////////////////////////////////////
// Template pointer array that deletes its elements in destructor
template <class T>
class CSafePtrArray : public CTypedPtrArray<CPtrArray,T*>
{
public:
    CSafePtrArray(BOOL bDeleteContent = TRUE) 
        : m_bDeleteContentInDestr(bDeleteContent)
    {
    }
    ~CSafePtrArray()
    {
        if (m_bDeleteContentInDestr)
            Reset();
    }
    void Reset()
    {
        for(int i=0;i<GetSize();i++)
            delete (*this)[i];
        RemoveAll();
    }
    T* operator[](int nIndex) const
    { 
        return CTypedPtrArray<CPtrArray,T*>::operator[](nIndex);
    }
    T*& operator[](int nIndex)
    { 
        return CTypedPtrArray<CPtrArray,T*>::operator[](nIndex);
    }
protected:
    BOOL m_bDeleteContentInDestr;
};

//////////////////////////////////////////////////////////////////
// Template pointer array that deletes its elements in destructor
template <class T>
class CSafeObjPtrArray : public CTypedPtrArray<CObArray,T*>
{
public:
    CSafeObjPtrArray(BOOL bDeleteContent = TRUE) 
        : m_bDeleteContentInDestr(bDeleteContent)
    {
    }
    ~CSafeObjPtrArray()
    {
        if (m_bDeleteContentInDestr)
            Reset();
    }
    void Reset()
    {
        for(int i=0;i<GetSize();i++)
            delete (*this)[i];
        RemoveAll();
    }
    T *operator[](int nIndex) const
    { 
        return CTypedPtrArray<CObArray,T*>::operator[](nIndex);
    }
    T*& operator[](int nIndex)
    { 
        return CTypedPtrArray<CObArray,T*>::operator[](nIndex);
    }
protected:
    BOOL m_bDeleteContentInDestr;
};

//////////////////////////////////////////////////////////////////
// Template pointer list that deletes its elements in destructor
template <class T>
class CSafePtrList : public CTypedPtrList<CPtrList,T*>
{
public:
    CSafePtrList(int nBlockSize = 10,BOOL bDeleteContent = TRUE) 
        : CTypedPtrList<CPtrList,T*>(nBlockSize)
          , m_bDeleteContentInDestr(bDeleteContent)
    {
    }
    ~CSafePtrList()
    {
        if (m_bDeleteContentInDestr)
            Reset();
    }
    void Reset()
    {
        while(!IsEmpty())  
            delete RemoveHead();
    }
protected:
    BOOL m_bDeleteContentInDestr;
};

//////////////////////////////////////////////////////////////////
// Template pointer list that deletes its elements in destructor
template <class T>
class CSafeObjPtrList : public CTypedPtrList<CObList,T*>
{
public:
    CSafeObjPtrList(int nBlockSize = 10,BOOL bDeleteContent = TRUE)
        : CTypedPtrList<CObList,T*>(nBlockSize)
          , m_bDeleteContentInDestr(bDeleteContent)
    {
    }
    ~CSafeObjPtrList()
    {
        if (m_bDeleteContentInDestr)
            Reset();
    }
    void Reset()
    {
        while(!IsEmpty())  
            delete RemoveHead();
    }
protected:
    BOOL m_bDeleteContentInDestr;
};

//////////////////////////////////////////////////////////////////
// Template pointer map that deletes its elements in destructor
template <class Key,class Value>
class CSafePtrMap : public CMap<Key,Key&,Value*,Value*&>
{
public:
    CSafePtrMap(BOOL bDeleteContent = TRUE) 
        : m_bDeleteContentInDestr(bDeleteContent)
    {
    }
    ~CSafePtrMap()
    {
        if (m_bDeleteContentInDestr)
            Reset();
    }
    void Reset()
    {
        POSITION pos = GetStartPosition();
        while(pos)
        {
            Key key;
            Value *Val=0;
            GetNextAssoc(pos,key, Val);
            delete Val;
        }
        RemoveAll();   
    }  
    Value*& operator[](Key key) 
    { 
        return CMap<Key,Key&,Value*,Value*&>::operator[](key);
    }  
protected:
    BOOL m_bDeleteContentInDestr;
};

#endif

// End of File
